package com.plugin.coolbus

import android.app.Activity
import android.app.Dialog
import androidx.annotation.NonNull;
import com.auth0.android.Auth0
import com.auth0.android.Auth0Exception
import com.auth0.android.authentication.AuthenticationException
import com.auth0.android.provider.AuthCallback
import com.auth0.android.provider.VoidCallback
import com.auth0.android.provider.WebAuthProvider
import com.auth0.android.result.Credentials

import io.flutter.embedding.engine.plugins.FlutterPlugin
import io.flutter.embedding.engine.plugins.activity.ActivityAware
import io.flutter.embedding.engine.plugins.activity.ActivityPluginBinding
import io.flutter.plugin.common.MethodCall
import io.flutter.plugin.common.MethodChannel
import io.flutter.plugin.common.MethodChannel.MethodCallHandler
import io.flutter.plugin.common.MethodChannel.Result
import io.flutter.plugin.common.PluginRegistry.Registrar

/** CoolbusPlugin */
public class CoolbusPlugin : FlutterPlugin, MethodCallHandler, ActivityAware {
    /// The MethodChannel that will the communication between Flutter and native Android
    ///
    /// This local reference serves to register the plugin with the Flutter Engine and unregister it
    /// when the Flutter Engine is detached from the Activity
    private lateinit var channel: MethodChannel
    private lateinit var auth0: Auth0
    private var activity: Activity? = null

    override fun onAttachedToActivity(binding: ActivityPluginBinding) {
        binding.activity.let {
            this.activity = binding.activity
            auth0 = Auth0(it);
            auth0.isOIDCConformant = true;
        }
    }

    override fun onDetachedFromActivity() {
        activity = null
    }

    override fun onDetachedFromActivityForConfigChanges() {

    }

    override fun onReattachedToActivityForConfigChanges(binding: ActivityPluginBinding) {

    }

    override fun onAttachedToEngine(@NonNull flutterPluginBinding: FlutterPlugin.FlutterPluginBinding) {
        channel = MethodChannel(flutterPluginBinding.getFlutterEngine().getDartExecutor(), "coolbus")
        channel.setMethodCallHandler(this);
    }

    // This static function is optional and equivalent to onAttachedToEngine. It supports the old
    // pre-Flutter-1.12 Android projects. You are encouraged to continue supporting
    // plugin registration via this function while apps migrate to use the new Android APIs
    // post-flutter-1.12 via https://flutter.dev/go/android-project-migration.
    //
    // It is encouraged to share logic between onAttachedToEngine and registerWith to keep
    // them functionally equivalent. Only one of onAttachedToEngine or registerWith will be called
    // depending on the user's project. onAttachedToEngine or registerWith must both be defined
    // in the same class.
    companion object {
        @JvmStatic
        fun registerWith(registrar: Registrar) {
            val channel = MethodChannel(registrar.messenger(), "coolbus")
            channel.setMethodCallHandler(CoolbusPlugin())
        }
    }

    override fun onMethodCall(@NonNull call: MethodCall, @NonNull result: Result) {
        if (call.method == "getPlatformVersion") {
            result.success("Android ${android.os.Build.VERSION.RELEASE}")
        } else if (call.method == "login") {
            login(result)
        } else if (call.method == "logout") {
            logout(result)
        } else if (call.method == "refresh") {
            //refresh(result)
        } else {
            result.notImplemented()
        }
    }

    override fun onDetachedFromEngine(@NonNull binding: FlutterPlugin.FlutterPluginBinding) {
        channel.setMethodCallHandler(null)
    }

    private fun login(result: Result) {
        activity?.let {
            WebAuthProvider.login(auth0)
                    .withScheme("demo")
                    .withAudience("https://pkpdeveloper.auth0.com/userinfo")
                    .start(it, object : AuthCallback {
                        override fun onSuccess(credentials: Credentials) {
                            val map = mutableMapOf<String, String?>()
                            map["idToken"] = credentials.idToken
                            map["accessToken"] = credentials.accessToken
                            map["refreshToken"] = credentials.refreshToken
                            map["scope"] = credentials.scope
                            map["type"] = credentials.type
                            map["expiresAt"] = credentials.expiresAt.toString()
                            map["expiresIn"] = credentials.expiresIn.toString()
                            it.runOnUiThread { result.success(map) }
                        }

                        override fun onFailure(dialog: Dialog) {
                        }

                        override fun onFailure(exception: AuthenticationException?) {
                            it.runOnUiThread {
                                result.error(exception?.code, exception?.message, null)
                            }
                        }

                    })
        }
    }

    private fun logout(result: Result) {
        activity?.let {
            WebAuthProvider.logout(auth0)
                    .withScheme("demo")
                    .start(it, object : VoidCallback {
                        override fun onSuccess(payload: Void?) {
                            it.runOnUiThread {
                                result.success(true)

                            }
                        }

                        override fun onFailure(error: Auth0Exception?) {
                            it.runOnUiThread {
                                result.error("500", error?.message, null)
                            }
                        }

                    })
        }
    }
}
