import Flutter
import UIKit
import Auth0

public class SwiftCoolbusPlugin: NSObject, FlutterPlugin{
    public static func register(with registrar: FlutterPluginRegistrar) {
        let channel = FlutterMethodChannel(name: "coolbus", binaryMessenger: registrar.messenger())
        let instance = SwiftCoolbusPlugin()
        registrar.addMethodCallDelegate(instance, channel: channel)
    }
    
    public func handle(_ call: FlutterMethodCall, result: @escaping FlutterResult) {
        if(call.method == "login") {
            login(result: result)
        } else if(call.method == "logout"){
            logout(result: result)
        }
    }
    
    private func login(result: @escaping FlutterResult){
        Auth0
            .webAuth()
            .scope("openid profile")
            .audience("https://pkpdeveloper.auth0.com/userinfo")
            .start {
                switch $0 {
                case .failure(let error):
                    result(FlutterError.init(code: "403", message: error.localizedDescription, details: nil))
                case .success(let credentials):
                    var resultMap = [String : String]()
                    resultMap["idToken"] = credentials.idToken
                    resultMap["accessToken"] = credentials.accessToken
                    resultMap["tokenType"] = credentials.tokenType
                    resultMap["scope"] = credentials.scope
                    resultMap["expiresIn"] = credentials.expiresIn?.description
                    
                   result(resultMap)
                }
        }
    }
    
    private func logout(result: @escaping FlutterResult){
        Auth0
            .webAuth()
            .clearSession(federated:false) {
                switch $0 {
                case true:
                    result(true)
                case false:
                    result(FlutterError.init(code: "500", message: "Auth Logout error", details: nil))
                }
        }
    }
}
